@@ -205,7 +205,6 @@ var Notebook = GObject.registerClass({
     scrollToBottom(index) {
         let tabData = this._tabs[index];
         tabData._scrollToBottom = true;
-
     }
 
     _onAdjustValueChanged(tabData) {
@@ -482,13 +481,16 @@ class RedBorderEffect extends Clutter.Effect {
         this._pipeline = null;
     }
 
-    vfunc_paint(paintContext) {
-        let framebuffer = paintContext.get_framebuffer();
-        let coglContext = framebuffer.get_context();
+    vfunc_paint_node(node, paintContext) {
         let actor = this.get_actor();
-        actor.continue_paint(paintContext);
+
+        const actorNode = new Clutter.ActorNode(actor, -1);
+        node.add_child(actorNode);
 
         if (!this._pipeline) {
+            const framebuffer = paintContext.get_framebuffer();
+            const coglContext = framebuffer.get_context();
+
             let color = new Cogl.Color();
             color.init_from_4ub(0xff, 0, 0, 0xc4);
 
@@ -499,18 +501,28 @@ class RedBorderEffect extends Clutter.Effect {
         let alloc = actor.get_allocation_box();
         let width = 2;
 
+        const pipelineNode = new Clutter.PipelineNode(this._pipeline);
+        pipelineNode.set_name('Red Border');
+        node.add_child(pipelineNode);
+
+        const box = new Clutter.ActorBox();
+
         // clockwise order
-        framebuffer.draw_rectangle(this._pipeline,
-            0, 0, alloc.get_width(), width);
-        framebuffer.draw_rectangle(this._pipeline,
-            alloc.get_width() - width, width,
-            alloc.get_width(), alloc.get_height());
-        framebuffer.draw_rectangle(this._pipeline,
-            0, alloc.get_height(),
-            alloc.get_width() - width, alloc.get_height() - width);
-        framebuffer.draw_rectangle(this._pipeline,
-            0, alloc.get_height() - width,
-            width, width);
+        box.set_origin(0, 0);
+        box.set_size(alloc.get_width(), width);
+        pipelineNode.add_rectangle(box);
+
+        box.set_origin(alloc.get_width() - width, width);
+        box.set_size(width, alloc.get_height() - width);
+        pipelineNode.add_rectangle(box);
+
+        box.set_origin(0, alloc.get_height() - width);
+        box.set_size(alloc.get_width() - width, width);
+        pipelineNode.add_rectangle(box);
+
+        box.set_origin(0, width);
+        box.set_size(width, alloc.get_height() - width * 2);
+        pipelineNode.add_rectangle(box);
     }
 });
 
@@ -688,7 +700,10 @@ var Extensions = GObject.registerClass({
             this._extensionsList.remove_actor(this._noExtensions);
 
         this._numExtensions++;
-        this._extensionsList.add(extensionDisplay);
+        const { name } = extension.metadata;
+        const pos = [...this._extensionsList].findIndex(
+            dsp => dsp._extension.metadata.name.localeCompare(name) > 0);
+        this._extensionsList.insert_child_at_index(extensionDisplay, pos);
     }
 
     _onViewSource(actor) {
@@ -751,6 +766,7 @@ var Extensions = GObject.registerClass({
 
     _createExtensionDisplay(extension) {
         let box = new St.BoxLayout({ style_class: 'lg-extension', vertical: true });
+        box._extension = extension;
         let name = new St.Label({
             style_class: 'lg-extension-name',
             text: extension.metadata.name,
@@ -1113,10 +1129,6 @@ class LookingGlass extends St.BoxLayout {
             // Ensure we don't get newlines in the command; the history file is
             // newline-separated.
             text = text.replace('\n', ' ');
-            // Strip leading and trailing whitespace
-            text = text.replace(/^\s+/g, '').replace(/\s+$/g, '');
-            if (text == '')
-                return true;
             this._evaluate(text);
             return true;
         });
@@ -1224,7 +1236,9 @@ class LookingGlass extends St.BoxLayout {
     }
 
     _evaluate(command) {
-        this._history.addItem(command);
+        command = this._history.addItem(command); // trims command
+        if (!command)
+            return;
 
         let lines = command.split(';');
         lines.push('return %s'.format(lines.pop()));
