@@ -7,7 +7,6 @@ const Signals = imports.signals;
 const Background = imports.ui.background;
 const FocusCaretTracker = imports.ui.focusCaretTracker;
 const Main = imports.ui.main;
-const MagnifierDBus = imports.ui.magnifierDBus;
 const Params = imports.misc.params;
 const PointerWatcher = imports.ui.pointerWatcher;
 
@@ -100,12 +99,15 @@ var Magnifier = class Magnifier {
         this._mouseSprite = new Clutter.Actor({ request_mode: Clutter.RequestMode.CONTENT_SIZE });
         this._mouseSprite.content = new MouseSpriteContent();
 
+        this._cursorRoot = new Clutter.Actor();
+        this._cursorRoot.add_actor(this._mouseSprite);
+
         // Create the first ZoomRegion and initialize it according to the
         // magnification settings.
 
         [this.xMouse, this.yMouse] = global.get_pointer();
 
-        let aZoomRegion = new ZoomRegion(this, this._mouseSprite);
+        let aZoomRegion = new ZoomRegion(this, this._cursorRoot);
         this._zoomRegions.push(aZoomRegion);
         this._settingsInit(aZoomRegion);
         aZoomRegion.scrollContentsTo(this.xMouse, this.yMouse);
@@ -114,8 +116,6 @@ var Magnifier = class Magnifier {
             this.setActive(St.Settings.get().magnifier_active);
         });
 
-        // Export to dbus.
-        new MagnifierDBus.ShellMagnifier();
         this.setActive(St.Settings.get().magnifier_active);
     }
 
@@ -172,9 +172,6 @@ var Magnifier = class Magnifier {
             this.stopTrackingMouse();
         }
 
-        if (this._crossHairs)
-            this._crossHairs.setEnabled(activate);
-
         // Make sure system mouse pointer is shown when all zoom regions are
         // invisible.
         if (!activate)
@@ -269,7 +266,7 @@ var Magnifier = class Magnifier {
      * @returns {ZoomRegion} the newly created ZoomRegion.
      */
     createZoomRegion(xMagFactor, yMagFactor, roi, viewPort) {
-        let zoomRegion = new ZoomRegion(this, this._mouseSprite);
+        let zoomRegion = new ZoomRegion(this, this._cursorRoot);
         zoomRegion.setViewPort(viewPort);
 
         // We ignore the redundant width/height on the ROI
@@ -830,12 +827,6 @@ var ZoomRegion = class ZoomRegion {
         let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
         let [xCaret, yCaret] = [extents.x * scaleFactor, extents.y * scaleFactor];
 
-        // Ignore event(s) if the caret size is none (0x0). This happens a lot if
-        // the cursor offset can't be translated into a location. This is a work
-        // around.
-        if (extents.width === 0 && extents.height === 0)
-            return;
-
         if (this._xCaret !== xCaret || this._yCaret !== yCaret) {
             [this._xCaret, this._yCaret] = [xCaret, yCaret];
             this._centerFromCaretPosition();
@@ -1175,8 +1166,6 @@ var ZoomRegion = class ZoomRegion {
                 this._yDelayed = null;
             }
 
-            this._scrollContentsTimerId = 0;
-
             return GLib.SOURCE_REMOVE;
         });
 
@@ -1664,7 +1653,6 @@ var ZoomRegion = class ZoomRegion {
 var Crosshairs = GObject.registerClass(
 class Crosshairs extends Clutter.Actor {
     _init() {
-
         // Set the group containing the crosshairs to three times the desktop
         // size in case the crosshairs need to appear to be infinite in
         // length (i.e., extend beyond the edges of the view they appear in).
@@ -1687,7 +1675,9 @@ class Crosshairs extends Clutter.Actor {
         this._clipSize = [0, 0];
         this._clones = [];
         this.reCenter();
-        this._monitorsChangedId = 0;
+
+        Main.layoutManager.connect('monitors-changed',
+                                   this._monitorsChanged.bind(this));
     }
 
     _monitorsChanged() {
@@ -1695,16 +1685,6 @@ class Crosshairs extends Clutter.Actor {
         this.reCenter();
     }
 
-    setEnabled(enabled) {
-        if (enabled && this._monitorsChangedId === 0) {
-            this._monitorsChangedId = Main.layoutManager.connect(
-                'monitors-changed', this._monitorsChanged.bind(this));
-        } else if (!enabled && this._monitorsChangedId !== 0) {
-            Main.layoutManager.disconnect(this._monitorsChangedId);
-            this._monitorsChangedId = 0;
-        }
-    }
-
     /**
     * addToZoomRegion
     * Either add the crosshairs actor to the given ZoomRegion, or, if it is
